﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using System.Security.Principal;
using System.Security.Permissions;
using System.ServiceModel.Activation;
using System.Threading;
using System.Web;

namespace InteLect.Services
{
    /// <summary>
    /// Implementación del servicio web para InteLect Administración Web
    /// </summary>
    public class InteLectAdministracion : IInteLectAdministracion
    {
        private InteLectDataContext m_DataContext = new InteLectDataContext();

        #region Usuarios

        /// <summary>
        /// Obtiene el listado de usuarios
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de usuarios</returns>
        public IEnumerable<Usuario> ListUsers(Administrativo user)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (administrativo != null)
            {
                return this.m_DataContext.Usuario.ToArray();
            }
            return new Usuario[0];
        }

        /// <summary>
        /// Obtiene un usuario por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="userId">Identificador del usuario</param>
        /// <returns>El usuario</returns>
        public Usuario GetUser(Administrativo user, Guid userId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (administrativo != null)
            {
                return this.m_DataContext.Usuario.SingleOrDefault(entity => entity.UsuarioId == userId);
            }
            return null;
        }

        /// <summary>
        /// Autentica un administrativo
        /// </summary>
        /// <param name="username">Nombre de usuario</param>
        /// <param name="password">Contraseña</param>
        /// <returns>El administrativo autenticado</returns>
        public Administrativo AuthenticateUser(string username, string password)
        {
            var administrativo = this.m_DataContext.Usuario.OfType<Administrativo>().SingleOrDefault(entity => entity.NombreUsuario == username);
            if (administrativo != null &&
                administrativo.Contraseña == password &&
                administrativo.FechaBaja == null)
            {
                return administrativo;
            }
            return null;
        }

        /// <summary>
        /// Cambia la contraseña de un administrativo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="newPassword">Nueva contraseña</param>
        /// <returns>Un valor indicando si ha sido posible cambiar la contraseña del administrativo o no</returns>
        public bool ChangeUserPassword(Administrativo user, string newPassword)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (administrativo != null)
            {
                var usuario = this.m_DataContext.Usuario.Single(entity => entity.UsuarioId == administrativo.UsuarioId);
                usuario.Contraseña = newPassword;
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        #endregion

        #region Compañías

        /// <summary>
        /// Obtiene el listado de compañías
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de compañías</returns>
        public IEnumerable<Compañia> ListCompanies(Administrativo user)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Compañia.ToArray();
            }
            return new Compañia[0];
        }

        /// <summary>
        /// Obtiene una compañía por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <returns>La compañía</returns>
        public Compañia GetCompany(Administrativo user, Guid companyId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Compañia.SingleOrDefault(entity => entity.CompañiaId == companyId);
            }
            return null;
        }

        /// <summary>
        /// Actualiza los datos de una compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="company">Compañía</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la compañía o no</returns>
        public bool UpdateCompany(Administrativo user, Compañia company)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var compañia = this.m_DataContext.Compañia.SingleOrDefault(entity => entity.CompañiaId == company.CompañiaId);
                if (compañia != null)
                {
                    compañia.Codigo = company.Codigo;
                    compañia.Nombre = company.Nombre;
                    compañia.FechaBaja = company.FechaBaja;
                    compañia.Comentarios = company.Comentarios;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Crea una nueva compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="company">Compañía</param>
        /// <returns>Un valor indicando si ha sido posible crear la compañía o no</returns>
        public bool AddCompany(Administrativo user, Compañia company)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                company.CompañiaId = Guid.NewGuid();
                company.FechaAlta = DateTime.Today;
                this.m_DataContext.Compañia.InsertOnSubmit(company);
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        /// <summary>
        /// Da de baja una compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la compañía o no</returns>
        public bool DeleteCompany(Administrativo user, Guid companyId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var compañia = this.m_DataContext.Compañia.SingleOrDefault(entity => entity.CompañiaId == companyId);
                if (compañia != null)
                {
                    compañia.FechaBaja = DateTime.Today;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region Medidas

        /// <summary>
        /// Obtiene el listado de medidas de una compañía
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <returns>El listado de medidas de la compañía</returns>
        public IEnumerable<Medida> ListMeasures(Administrativo user, Guid companyId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var compañia = this.m_DataContext.Compañia.SingleOrDefault(entity => entity.CompañiaId == companyId);
                if (compañia != null)
                {
                    return compañia.Medida.ToArray();
                }
            }
            return new Medida[0];
        }

        /// <summary>
        /// Obtiene una medida por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measureId">Identificador de la medida</param>
        /// <returns>La medida</returns>
        public Medida GetMeasure(Administrativo user, Guid measureId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Medida.SingleOrDefault(entity => entity.MedidaId == measureId);
            }
            return null;
        }

        /// <summary>
        /// Actualiza los datos de una medida
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measure">Medida</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la medida o no</returns>
        public bool UpdateMeasure(Administrativo user, Medida measure)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var medida = this.m_DataContext.Medida.SingleOrDefault(entity => entity.MedidaId == measure.MedidaId);
                if (medida != null)
                {
                    medida.Codigo = measure.Codigo;
                    medida.Nombre = measure.Nombre;
                    medida.Unidades = measure.Unidades;
                    medida.FechaBaja = measure.FechaBaja;
                    medida.Comentarios = measure.Comentarios;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Crea una nueva medida
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="companyId">Identificador de la compañía</param>
        /// <param name="measure">Medida</param>
        /// <returns>Un valor indicando si ha sido posible crear la medida o no</returns>
        public bool AddMeasure(Administrativo user, Guid companyId, Medida measure)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                measure.MedidaId = Guid.NewGuid();
                measure.CompañiaId = companyId;
                measure.FechaAlta = DateTime.Today;
                this.m_DataContext.Medida.InsertOnSubmit(measure);
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        /// <summary>
        /// Da de baja una medida
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measureId">Identificador de la medida</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la medida o no</returns>
        public bool DeleteMeasure(Administrativo user, Guid measureId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var medida = this.m_DataContext.Medida.SingleOrDefault(entity => entity.MedidaId == measureId);
                if (medida != null)
                {
                    medida.FechaBaja = DateTime.Today;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region Clientes

        /// <summary>
        /// Obtiene el listado de clientes
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de clientes</returns>
        public IEnumerable<Cliente> ListClients(Administrativo user)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Cliente.ToArray();
            }
            return new Cliente[0];
        }

        /// <summary>
        /// Obtiene un cliente por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <returns>El cliente</returns>
        public Cliente GetClient(Administrativo user, Guid clientId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Cliente.SingleOrDefault(entity => entity.ClienteId == clientId);
            }
            return null;
        }

        /// <summary>
        /// Actualiza los datos de un cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="client">Cliente</param>
        /// <returns>Un valor indicando si ha sido posible actualizar el cliente o no</returns>
        public bool UpdateClient(Administrativo user, Cliente client)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var cliente = this.m_DataContext.Cliente.SingleOrDefault(entity => entity.ClienteId == client.ClienteId);
                if (cliente != null)
                {
                    cliente.Codigo = client.Codigo;
                    cliente.Nombre = client.Nombre;
                    cliente.Apellidos = client.Apellidos;
                    cliente.FechaBaja = client.FechaBaja;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Crea un nuevo cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="client">Cliente</param>
        /// <returns>Un valor indicando si ha sido posible crear el cliente o no</returns>
        public bool AddClient(Administrativo user, Cliente client)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                client.ClienteId = Guid.NewGuid();
                client.FechaAlta = DateTime.Today;
                this.m_DataContext.Cliente.InsertOnSubmit(client);
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        /// <summary>
        /// Da de baja un cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja el cliente o no</returns>
        public bool DeleteClient(Administrativo user, Guid clientId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var client = this.m_DataContext.Cliente.SingleOrDefault(entity => entity.ClienteId == clientId);
                if (client != null)
                {
                    client.FechaBaja = DateTime.Today;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region Direcciones

        /// <summary>
        /// Obtiene el listado de direcciones de un cliente
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <returns>El listado de direcciones del cliente</returns>
        public IEnumerable<Direccion> ListAddresses(Administrativo user, Guid clientId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var cliente = this.m_DataContext.Cliente.SingleOrDefault(entity => entity.ClienteId == clientId);
                if (cliente != null)
                {
                    return cliente.Direccion.ToArray();
                }
            }
            return new Direccion[0];
        }

        /// <summary>
        /// Obtiene una dirección por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="addressId">Identificador de la dirección</param>
        /// <returns>La dirección</returns>
        public Direccion GetAddress(Administrativo user, Guid addressId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Direccion.SingleOrDefault(entity => entity.DireccionId == addressId);
            }
            return null;
        }

        /// <summary>
        /// Actualiza los datos de una dirección
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="address">Dirección</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la dirección o no</returns>
        public bool UpdateAddress(Administrativo user, Direccion address)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var direccion = this.m_DataContext.Direccion.SingleOrDefault(entity => entity.DireccionId == address.DireccionId);
                if (direccion != null)
                {
                    direccion.Codigo = address.Codigo;
                    direccion.Nombre = address.Nombre;
                    direccion.CodigoPostal = address.CodigoPostal;
                    direccion.Poblacion = address.Poblacion;
                    direccion.Provincia = address.Provincia;
                    direccion.FechaBaja = address.FechaBaja;
                    direccion.Comentarios = address.Comentarios;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Crea una nueva dirección
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="clientId">Identificador del cliente</param>
        /// <param name="address">Dirección</param>
        /// <returns>Un valor indicando si ha sido posible crear la dirección o no</returns>
        public bool AddAddress(Administrativo user, Guid clientId, Direccion address)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                address.DireccionId = Guid.NewGuid();
                address.ClienteId = clientId;
                address.FechaAlta = DateTime.Today;
                this.m_DataContext.Direccion.InsertOnSubmit(address);
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        /// <summary>
        /// Da de baja una dirección
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="addressId">Identificador de la dirección</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la dirección o no</returns>
        public bool DeleteAddress(Administrativo user, Guid addressId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var direccion = this.m_DataContext.Direccion.SingleOrDefault(entity => entity.DireccionId == addressId);
                if (direccion != null)
                {
                    direccion.FechaBaja = DateTime.Today;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region Servicios

        /// <summary>
        /// Obtiene el listado de servicios
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de servicios</returns>
        public IEnumerable<Servicio> ListServices(Administrativo user)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Servicio.ToArray();
            }
            return new Servicio[0];
        }

        /// <summary>
        /// Obtiene un servicio por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="serviceId">Identificador del servicio</param>
        /// <returns>El servicio</returns>
        public Servicio GetService(Administrativo user, Guid serviceId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.Servicio.SingleOrDefault(entity => entity.ServicioId == serviceId);
            }
            return null;
        }

        /// <summary>
        /// Actualiza los datos de un servicio
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="service">Servicio</param>
        /// <returns>Un valor indicando si ha sido posible actualizar el servicio o no</returns>
        public bool UpdateService(Administrativo user, Servicio service)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var servicio = this.m_DataContext.Servicio.SingleOrDefault(entity => entity.ServicioId == service.ServicioId);
                if (servicio != null)
                {
                    servicio.Codigo = service.Codigo;
                    servicio.FechaBaja = service.FechaBaja;
                    servicio.Comentarios = service.Comentarios;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Crea un nuevo servicio
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="measureId">Identificador de la medida</param>
        /// <param name="addressId">Identificador de la dirección</param>
        /// <param name="service">Servicio</param>
        /// <returns>Un valor indicando si ha sido posible crear el servicio o no</returns>
        public bool AddService(Administrativo user, Guid measureId, Guid addressId, Servicio service)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                service.ServicioId = Guid.NewGuid();
                service.MedidaId = measureId;
                service.DireccionId = addressId;
                service.FechaAlta = DateTime.Today;
                this.m_DataContext.Servicio.InsertOnSubmit(service);
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        /// <summary>
        /// Da de baja un servicio
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="serviceId">Identificador del servicio</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja el servicio o no</returns>
        public bool DeleteService(Administrativo user, Guid serviceId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var servicio = this.m_DataContext.Servicio.SingleOrDefault(entity => entity.ServicioId == serviceId);
                if (servicio != null)
                {
                    servicio.FechaBaja = DateTime.Today;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region Órdenes de trabajo

        /// <summary>
        /// Obtiene el listado de órdenes de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <returns>El listado de órdenes de trabajo</returns>
        public IEnumerable<OrdenTrabajo> ListWorkOrders(Administrativo user)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.OrdenTrabajo.ToArray();
            }
            return new OrdenTrabajo[0];
        }

        /// <summary>
        /// Obtiene una órden de trabajo por su identificador
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="workOrderId">Identificador de la órden de trabajo</param>
        /// <returns>La órden de trabajo</returns>
        public OrdenTrabajo GetWorkOrder(Administrativo user, Guid workOrderId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                return this.m_DataContext.OrdenTrabajo.SingleOrDefault(entity => entity.OrdenTrabajoId == workOrderId);
            }
            return null;
        }

        /// <summary>
        /// Actualiza los datos de una órden de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="workOrder">Órden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la órden de trabajo o no</returns>
        public bool UpdateWorkOrder(Administrativo user, OrdenTrabajo workOrder)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var ordenTrabajo = this.m_DataContext.OrdenTrabajo.SingleOrDefault(entity => entity.OrdenTrabajoId == workOrder.OrdenTrabajoId);
                if (ordenTrabajo != null)
                {
                    ordenTrabajo.Fecha = workOrder.Fecha;
                    ordenTrabajo.FechaLectura = workOrder.FechaLectura;
                    ordenTrabajo.Cantidad = workOrder.Cantidad;
                    ordenTrabajo.Comentarios = workOrder.Comentarios;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Crea una nueva órden de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="inspectorId">Identificador del inspector</param>
        /// <param name="serviceId">Identificador del servicio</param>
        /// <param name="workOrder">Órden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible crear la órden de trabajo o no</returns>
        public bool AddWorkOrder(Administrativo user, Guid inspectorId, Guid serviceId, OrdenTrabajo workOrder)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                workOrder.OrdenTrabajoId = Guid.NewGuid();
                workOrder.UsuarioId = inspectorId;
                this.m_DataContext.OrdenTrabajo.InsertOnSubmit(workOrder);
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        /// <summary>
        /// Elimina una órden de trabajo
        /// </summary>
        /// <param name="user">Administrativo</param>
        /// <param name="workOrderId">Identificador de la órden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible dar de baja la órden de trabajo o no</returns>
        public bool DeleteWorkOrder(Administrativo user, Guid workOrderId)
        {
            var administrativo = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (user != null)
            {
                var workOrder = this.m_DataContext.OrdenTrabajo.SingleOrDefault(entity => entity.OrdenTrabajoId == workOrderId);
                if (workOrder != null)
                {
                    this.m_DataContext.OrdenTrabajo.DeleteOnSubmit(workOrder);
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }

        #endregion
    }
}
