﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;

namespace InteLect.Services
{
    /// <summary>
    /// Implementación del servicio web para InteLect Inspección Móvil
    /// </summary>
    public class InteLectInspeccion : IInteLectInspeccion
    {
        private InteLectDataContext m_DataContext = new InteLectDataContext(); // Acceso a la base de datos

        /// <summary>
        /// Autentica un inspector
        /// </summary>
        /// <param name="username">Nombre de usuario</param>
        /// <param name="password">Contraseña</param>
        /// <returns>El inspector autenticado</returns>
        public UsuarioMovil AuthenticateUser(string username, string password)
        {
            var inspector = this.m_DataContext.Usuario.OfType<Inspector>().SingleOrDefault(entity => entity.NombreUsuario == username);
            if (inspector != null &&
                inspector.Contraseña == password &&
                inspector.FechaBaja == null)
            {
                return new UsuarioMovil()
                {
                    UsuarioId = inspector.UsuarioId,
                    Codigo = inspector.Codigo,
                    Nombre = inspector.Nombre,
                    Apellidos = inspector.Apellidos,
                    NombreUsuario = username,
                    Contraseña = password
                };
            }
            return null;
        }

        /// <summary>
        /// Cambia la contraseña de un inspector
        /// </summary>
        /// <param name="user">Inspector</param>
        /// <param name="newPassword">Nueva contraseña</param>
        /// <returns>Un valor indicando si ha sido posible cambiar la contraseña del inspector o no</returns>
        public bool ChangeUserPassword(UsuarioMovil user, string newPassword)
        {
            var inspector = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (inspector != null)
            {
                var usuario = this.m_DataContext.Usuario.Single(entity => entity.UsuarioId == inspector.UsuarioId);
                usuario.Contraseña = newPassword;
                this.m_DataContext.SubmitChanges();
                return true;
            }
            return false;
        }

        /// <summary>
        /// Descarga el plan de trabajo de un inspector
        /// </summary>
        /// <param name="user">Inspector</param>
        /// <returns>El plan de trabajo del inspector</returns>
        public IEnumerable<OrdenTrabajoMovil> DownloadWorkPlan(UsuarioMovil user)
        {
            var workPlan = new List<OrdenTrabajoMovil>();
            var inspector = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (inspector != null)
            {
                var usuario = this.m_DataContext.Usuario.Single(entity => entity.UsuarioId == inspector.UsuarioId);
                foreach (var ordenTrabajo in usuario.OrdenTrabajo.Where(entity => entity.Fecha == DateTime.Today &&
                                                                                  entity.Cantidad == null))
                {
                    workPlan.Add(new OrdenTrabajoMovil()
                    {
                        OrdenTrabajoId = ordenTrabajo.OrdenTrabajoId,
                        UsuarioId = ordenTrabajo.UsuarioId,
                        Codigo = ordenTrabajo.Servicio.Codigo,
                        Fecha = ordenTrabajo.Fecha,
                        Compañia = ordenTrabajo.Servicio.Medida.Compañia.Nombre,
                        Medida = ordenTrabajo.Servicio.Medida.Nombre,
                        Cliente = string.Format("{0} {1}", ordenTrabajo.Servicio.Direccion.Cliente.Nombre, ordenTrabajo.Servicio.Direccion.Cliente.Apellidos),
                        Direccion = string.Format("{0}\n{1} - {2} ({3})", ordenTrabajo.Servicio.Direccion.Nombre, ordenTrabajo.Servicio.Direccion.CodigoPostal, ordenTrabajo.Servicio.Direccion.Poblacion, ordenTrabajo.Servicio.Direccion.Provincia)
                    });
                }
            }
            return workPlan;
        }

        /// <summary>
        /// Actualiza una orden de trabajo
        /// </summary>
        /// <param name="user">Inspector</param>
        /// <param name="workOrder">Orden de trabajo</param>
        /// <returns>Un valor indicando si ha sido posible actualizar la orden de trabajo o no</returns>
        public bool UpdateWorkOrder(UsuarioMovil user, OrdenTrabajoMovil workOrder)
        {
            var inspector = this.AuthenticateUser(user.NombreUsuario, user.Contraseña);
            if (inspector != null)
            {
                var ordenTrabajo = this.m_DataContext.OrdenTrabajo.SingleOrDefault(entity => entity.OrdenTrabajoId == workOrder.OrdenTrabajoId);
                if (ordenTrabajo != null &&
                    ordenTrabajo.FechaLectura == null &&
                    workOrder.Cantidad != null)
                {
                    ordenTrabajo.Cantidad = workOrder.Cantidad;
                    ordenTrabajo.FechaLectura = DateTime.Now;
                    this.m_DataContext.SubmitChanges();
                    return true;
                }
            }
            return false;
        }
    }

    [DataContract]
    public class UsuarioMovil
    {
        [DataMember]
        public Guid UsuarioId { get; set; }

        [DataMember]
        public string Codigo { get; set; }

        [DataMember]
        public string Nombre { get; set; }

        [DataMember]
        public string Apellidos { get; set; }

        [DataMember]
        public string NombreUsuario { get; set; }

        [DataMember]
        public string Contraseña { get; set; }
    }

    [DataContract]
    public class OrdenTrabajoMovil
    {
        [DataMember]
        public Guid OrdenTrabajoId { get; set; }

        [DataMember]
        public Guid UsuarioId { get; set; }

        [DataMember]
        public string Codigo { get; set; }

        [DataMember]
        public DateTime Fecha { get; set; }

        [DataMember]
        public Nullable<decimal> Cantidad { get; set; }

        [DataMember]
        public string Compañia { get; set; }

        [DataMember]
        public string Medida { get; set; }

        [DataMember]
        public string Cliente { get; set; }

        [DataMember]
        public string Direccion { get; set; }
    }
}
